C$Procedure      PLBORE_3 ( Plate Intersected by Ray )
 
      SUBROUTINE PLBORE_3 ( POSOBS, BSIGHT, PLID, BODY, XYZHIT, FOUND )
 
C$ Abstract
C
C     Determine the idcode of the plate and body fixed
C     coordinates of the point where an arbitrary ray intersects
C     the plate model.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      SHAPES
C
C$ Declarations
 
      IMPLICIT NONE
 
      DOUBLE PRECISION      POSOBS   ( 3 )
      DOUBLE PRECISION      BSIGHT   ( 3 )
      INTEGER               PLID
      INTEGER               BODY
      DOUBLE PRECISION      XYZHIT   ( 3 )
      LOGICAL               FOUND
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     POSOBS     I   observer location in body fixed coordinates
C     BSIGHT     I   ray direction in body fixed coordinates
C     PLID       O   ID code of the plate that lies along the boresight
C     BODY       O   body ID of the body that the plate belongs to
C     XYZHIT     O   point on plate model where ray hits
C     FOUND      O   TRUE if there is a plate, FALSE otherwise.
C
C$ Detailed_Input
C
C     POSOBS     observer location in body fixed coordinates
C
C     BSIGHT     ray direction from observer in body fixed coordinates
C
C
C$ Detailed_Output
C
C     PLID       is the plate ID of the first plate that is intersected
C                by the boresight of the instrument specified by INSTID.
C                In the event that the boresight intersects the surface
C                model at an edge or vertex, the plate whose normal is
C                closest to anti-parallel to the boresight direction is
C                selected.  If this criteria does not identify uniquely
C                the plate to chose, the plate whose center is closest
C                to the instrument is chose.  If this still does not
C                resolve which plate to chose, then the
C                plate with the smallest ID value (among those in
C                contention) is returned.
C
C     BODY       is the NAIF ID code for the body that the boresight
C                first intersects and to which the plate specified by
C                PLID belongs.
C
C     XYZHIT     is the closest point on the plate model where the ray
C                from the observer's location intersects.
C
C     FOUND      is a logical flag that indicates whether or not
C                the boresight vector does indeed intersect.  If
C                the boresight intersects a plate FOUND is TRUE.
C                Otherwise FOUND is FALSE.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If a required kernel (see Particulars) below cannot be located
C        an error will be diagnosed by one of the routines called
C        by this routine.
C
C$ Particulars
C
C     Given a set of objects whose surfaces are modeled by
C     a set of covering triangular plates, this routine returns
C     the first plate in the list of plates that is intersected
C     by the boresight of the specified instrument.
C
C     A substantial amount of preparation is required in the
C     users program before this routine can be called.
C
C     The user must "load" a shape model for the object of interest
C
C     Once these preparations have been performed this routine will
C     return the plate first intersected by the boresight (if one
C     exists.
C
C$ Examples
C
C     When time permits.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C
C-    SPICELIB Version 1.0.0, 25-FEB-1999 (JAB)
C
C        Based on PLBORE and PLBORE_2.
C
C-&
 
C$ Index_Entries
C
C     plate and plate model point intersected by ray
C
C-&
C
C     SPICELIB Functions
C
 
      DOUBLE PRECISION      VDOT
      DOUBLE PRECISION      VNORM
 
      LOGICAL               RETURN
      LOGICAL               FAILED
 
C
C     Parameters
C
 
C
C     Local Variables
C
 
 
      DOUBLE PRECISION      BDSITE ( 3 )
      DOUBLE PRECISION      CENTR  ( 3 )
      DOUBLE PRECISION      COSANG
      DOUBLE PRECISION      DIST
      DOUBLE PRECISION      EDGES  ( 3, 3      )
      DOUBLE PRECISION      NEAR
      DOUBLE PRECISION      NORML  ( 3 )
      DOUBLE PRECISION      OBSMAT ( 3, 3 )
      DOUBLE PRECISION      POINTS ( 3, 3 )
      DOUBLE PRECISION      SCALE
      DOUBLE PRECISION      TEMP
      DOUBLE PRECISION      TOCENT ( 3 )
 
 
      INTEGER               BCOUNT
      INTEGER               BODEX
      INTEGER               CBOD
      INTEGER               FSTIDX
      INTEGER               J
      INTEGER               LSTIDX
      INTEGER               NPLATE
      INTEGER               NVERT
      INTEGER               PRANGE ( 2 )
      INTEGER               TEMP1
      INTEGER               TEMP2
      INTEGER               TEST
      INTEGER               TMP
      INTEGER               VRANGE ( 2 )
 
      LOGICAL               HITS
      LOGICAL               HAVE
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'PLBORE_3' )
 
C
C     Until we have better knowledge we assume there is no intersection
C
      BODY  =  0
      PLID  =  0
      FOUND = .FALSE.
      HAVE  = .FALSE.
 
      NEAR = 9.D99
 
C
C     Obtain the unit vector of the ray from the observer.
C
 
      CALL VHAT   ( BSIGHT, BDSITE )
 
C
C     Determine the number of bodies that have plates
C     models loaded.
C
 
      CALL PLTOT ( BCOUNT, TEMP1, TEMP2 )
 
C
C     For each body, check the coverage with the field of view.
C
 
      DO BODEX = 1, BCOUNT
 
C
C        So far we haven't done any of the initialization work
C        for BOUNDS for this body.  Look up the body identifier
C        from the index
C
         CALL DX2BID ( BODEX, CBOD )
 
 
C
C        If something went wrong up above there is no point in
C        going on from here.
C
         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'PLBORE_3' )
            RETURN
         END IF
 
C
C        We are going to need to subtract the location of the observer
C        from vertices of a plate.  To speed things up a tiny bit,
C        we'll make 3 copies of the observer's location so that
C        we make a single subroutine call to handle the 3 subtractions.
C
         CALL VEQU ( POSOBS, OBSMAT(1,1) )
         CALL VEQU ( POSOBS, OBSMAT(1,2) )
         CALL VEQU ( POSOBS, OBSMAT(1,3) )
 
C
C        Now examine each plate for this body.
C
         CALL PLTBND ( BODEX, NPLATE, PRANGE, NVERT, VRANGE )
 
         FSTIDX = PRANGE(1)
         LSTIDX = PRANGE(2)
 
 
         DO J = FSTIDX, LSTIDX
 
C
C           Get the J'th plate center and normal.
C
            CALL NTHPLC ( J, CENTR )
            CALL NTHPLN ( J, NORML )
C
C           Compute the position of the observer relative to the
C           center of the plate.
C
            CALL VSUB ( POSOBS, CENTR, TOCENT )
C
C           Only if this plate is visible do we do anything else
C           with this plate.
C
            IF ( VDOT ( TOCENT, NORML ) .GT. 0.0D0 ) THEN
C
C
C              Retrieve the bodyfixed coordinates of the points that
C              are the vertices of the J'th plate. Compute the edges
C              of the tetrahedral angle having the observer as the apex
C              and the vertices as members of the edge rays.  Finally
C              see if the boresight lies inside the tetrahedral angle.
C
               CALL NTHPLV ( J,          POINTS            )
               CALL VSUBG  ( POINTS,     OBSMAT, 9,  EDGES )
               CALL INSANG ( BDSITE,
     .                       EDGES(1,1), EDGES(1,2), EDGES(1,3),
     .                       HITS,       SCALE )
 
C
C              If the boresight intersects this plate, see if we already
C              had an intersection.  If so see if this plate is closer
C              than the last intersection we had.
C
               IF ( HITS ) THEN
 
                  IF ( HAVE ) THEN
C
C                    If this intersection point is closer than
C                    the last one, this plate must be in front.
C
                     IF      ( SCALE .LT. NEAR ) THEN
 
                        BODY   = CBOD
                        NEAR   = SCALE
                        COSANG = VDOT ( BDSITE, NORML )
 
                        CALL NTHPLI ( J, TMP, PLID )
 
                     ELSE IF ( SCALE .EQ. NEAR ) THEN
C
C                       In this case we must have an edge point.
C                       We pick the plate that whose normal direction
C                       is closest to anti-parallel to the boresight
C                       direction.
C
                        CALL VHAT ( NORML, NORML )
 
                        TEMP = VDOT ( BDSITE, NORML )
 
                        IF ( TEMP .LT. COSANG ) THEN
 
                           COSANG = TEMP
                           BODY   = CBOD
                           DIST   = VNORM ( TOCENT )
 
                           CALL NTHPLI ( J, TMP, PLID )
 
                        ELSE IF ( TEMP .EQ. COSANG ) THEN
C
C                          Hmmmm.  We've got two parallel plates that
C                          share an edge or vertex.  Pick the one
C                          with the closest center.
C
                           TEMP = VNORM ( TOCENT )
 
                           IF      ( TEMP .LT. DIST ) THEN
 
                              DIST = TEMP
                              CALL NTHPLI ( J, TMP, PLID )
 
                           ELSE IF ( TEMP .EQ. DIST ) THEN
C
C                             The centers of the plates are equidistant.
C                             That's the end of the geometric tests.
C                             We just take the one that had the lower
C                             plate id number.
C
                              CALL NTHPLI ( J, TMP, TEST )
 
                              IF ( TEST .LT. PLID ) THEN
                                 PLID = TEST
                                 BODY = CBOD
                              END IF
 
                           END IF
 
                        END IF
 
                     END IF
 
                  ELSE
 
                     HAVE   = .TRUE.
                     NEAR   = SCALE
                     COSANG = VDOT ( BDSITE, NORML )
                     DIST   = VNORM( TOCENT )
                     BODY   = CBOD
 
                     CALL NTHPLI ( J, TMP, PLID )
 
                  END IF
 
               END IF
 
            END IF
C
C           Fetch the next plate for this body.
C
 
         END DO
 
C
C        We are done with the current body.  Do the same set of
C        steps for the next body in the list of bodies that have
C        plate models stored in the plate manager.
C
 
      END DO
 
C
C     We've done all of the tedious steps.  If we have an intersection
C     put the information into FOUND.
C
 
      FOUND = HAVE
 
C
C     Compute where the ray from the observer intersects the plate
C     model.
C
 
      IF ( FOUND ) THEN
         CALL VLCOM ( 1.D0, POSOBS, NEAR, BDSITE, XYZHIT )
      END IF
C
C     That's all folks
C
 
      CALL CHKOUT ( 'PLBORE_3' )
      RETURN
 
      END
